!---------------------------------------------------------------------------------------------------
! ReportPagesLostTest by Carl Barnes copyright 2022 released under the MIT License 
! based on ReportCloseSlowTest by Carl Barnes
!---------------------------------------------------------------------------------------------------
! To Test and create contention run it from a BAT file start multiple with /GO swict
!   START ReportPagesLostTest.exe /GO
!   START ReportPagesLostTest.exe /GO
!   START ReportPagesLostTest.exe /GO
!
! You can leave many Temp files and possibly cause problems for Reporting.
! Do that by Halt the test (button in Preview) or shutdown in Task Manger.
! Then use "View / Purge Temp Folder" button at the end to clean them out.
!---------------------------------------------------------------------------------------------------
! Posts say the Report is Duplicating some WMF names or losing them
!
! Description of ReportCloseSlowTest ... 
! I noticed when previewing large reports that after closing the preview it took a longgg time for focus to return to the calling window
! This program lets you test and time Close(Report) where the RTL deletes the WMF files in the Preview Queue
! versus calling API Delete File before Close(Report) to see its faster? 
! For me the API delete is 80x faster. E.g. 1000 pages RTL takes 13 seconds to close but API takes 0.15 seconds
!
! IMO RTL Remove() since C8 is far slower that it was previously. It changed from calling API DeleteFile to SHFileOperation()
! My guess is Close(Report) calls the RTL Remove().
! You can try CHECK('&Use RTL Remove()') to use REMOVE() instead of DeleteFile() and for me that is also 80x slower. Remove() is the problem.
!
! If you are concerned about what files this deletes search for DELETEFILE and REMOVE. You will see it only does Preview or Temp files.
! You can set " YES_Delete_Files (False) " below to not delete
!---------------------------------------------------------------------------------------------------
    PROGRAM
    INCLUDE('PRNPROP.CLW')
    INCLUDE('KeyCodes.CLW') 

YES_Delete_Files   EQUATE(1) !<--- When (1) DeleteFile() / Remove() actually happens, else outputs debug only

    MAP
CheckForPagesLOSTorDUP      PROCEDURE(QUEUE ReportPreviewQ),BOOL,PROC    !

Main        PROCEDURE()
PrintReport PROCEDURE(Long Pages2Print, BOOL UseApiDelete)
ReportPreviewer             PROCEDURE(QUEUE ReportPreviewQ, BOOL ForcePreview)
ReportCloseTempDeleteAPI    PROCEDURE(QUEUE RptPreviewQ)     ! Delete Preview files using API DeleteFile() before Close(Report)
ReportCloseTempRemoveRTL    PROCEDURE(QUEUE RptPreviewQ)     ! Delete Preview files using RTL Remove() before Close(Report) to prove Remove() is slow
TempFolderPurgeTmpFiles     PROCEDURE()
DB          PROCEDURE(STRING DebugMessage)
DBClear     PROCEDURE()     !Clear DebugView Buffer
Hex8        PROCEDURE(LONG LongInt),STRING

      MODULE('api')
        DeleteFile(*CSTRING lpFileName ),BOOL,PROC,RAW,PASCAL,DLL(1),name('DeleteFileA')        !Win API way to Delete a File
        GetTempPath(LONG nBufferLength,*CSTRING lpTempPath),LONG,PROC,PASCAL,DLL(1),RAW,NAME('GetTempPathA')
        OutputDebugString(*CSTRING cMsg),RAW,PASCAL,DLL(1),NAME('OutputDebugStringA')
        GetLastError(),LONG,PASCAL,DLL(1)
        GetCurrentProcessId(),LONG,PASCAL,DLL(1)
      END
    END

ResultsQ    QUEUE,PRE(RezQ)   !Results of tests to show in List
TimeOfDay       LONG          !RezQ:TimeOfDay 
Pages           LONG          !RezQ:Pages     
Rtl_Taken       STRING(11)    !RezQ:Rtl_Taken
Api_Taken       STRING(11)    !RezQ:Api_Taken
Comments        STRING(99)    !RezQ:Comments      
Rtl_Time        LONG          !RezQ:Rtl_Time
Api_Time        LONG          !RezQ:Api_Time
            END  
Last_Rtl_Time  LONG            
Last_Rtl_Pages LONG
ConfigINI      STRING('.\RptPagesLost.INI')
Glo:NoPreview  BYTE
Glo:AfterDeleteViewTemp  BYTE
Glo:NoApiDeleteUseRTL    BYTE       !Instead of API DeleteFile() to see if using REMOVE() is as slow as Close(Report) 
Glo:ProcessID            LONG       !Process ID on Reports to spot pages from other process
    CODE
    SYSTEM{PROP:PropVScroll}=1      !Make VScroll Thumb proportional
    SYSTEM{7A7Dh}=MSGMODE:CANCOPY   !PROP:MsgModeDefault
    Glo:ProcessID=GetCurrentProcessId()
    MAIN()
    RETURN

!================================================================================== Main ===========
Main   PROCEDURE    !-- Main Procedure - Window to Run Tests and Vioew Results --
!===================================================================================================
Pages2Print     LONG(500)
!        PROMPT('Posts have been made that Report Pages are being Duplicated or Lost on ClarionHub and Softvelocity.Clari' & |
!                'on.Reports. This generates simple reports then checks the PreviewQ does not have Duplicates and the WMF' & |
!                ' files exist.  It is based on "Report Close Slow" so there is some clutter.'),AT(9,3,339,29),USE(?PROMPT1), |
!                FONT(,9)
Window WINDOW('Report Pages Lost or Duplicated Test '),AT(,,358,200),CENTER,GRAY,IMM,SYSTEM,ICON(ICON:NoPrint), |
            FONT('Segoe UI',9),RESIZE
        PROMPT('Posts on ClarionHub and Softvelocity.Clarion.Reports indicate Report Pages are being Duplicated or Lost.' & |
                ' This test generates simple reports then checks the PreviewQ does not have Duplicates and the WMF files' & |
                ' exist and shows a Message(). It is based on "Report Close Slow" so there is some clutter.'),AT(9,3,339,25), |
                USE(?PROMPT1)
        PROMPT('Run with the /GO switch to run immediately to have 3+ running at the same time for some contention. You ' & |
                'can Halt the program in Report Preview or Task Manager to leave files in the TEMP folder to possibly ca' & |
                'sue problems.'),AT(9,30,339,18),USE(?PROMPT2)
        GROUP,AT(1,56),USE(?GrpLower)
            PROMPT('Test Pages:'),AT(5,59),USE(?Pages2Print:Pmt)
            ENTRY(@n5),AT(6,70,32,10),USE(Pages2Print)
            BUTTON('1. &RTL Normal<13,10>Close(Report) Test'),AT(51,58,65,24),USE(?ReportNormalRTL),DISABLE
            BUTTON('2. &API DeleteFile<13,10>Close(Report) Test'),AT(128,58,65,24),USE(?ReportDeleteAPI)
            CHECK('No &Preview'),AT(51,84,,10),USE(Glo:NoPreview),SKIP,TIP('Faster Testing<13,10>Run Report and close wi' & |
                    'thout Preview')
            CHECK('After &Delete, before Close, View Temp Folder'),AT(128,84,,10),USE(Glo:AfterDeleteViewTemp),SKIP, |
                    TIP('After API DeleteFile() but before Close(Report)<13,10>open Temp Folder View to confirm. Screws ' & |
                    'up timing.')
            CHECK('&Use RTL Remove()'),AT(197,65,,10),USE(Glo:NoApiDeleteUseRTL),SKIP,DISABLE,TIP('Check box to use RTL ' & |
                    'Remove() instead of API DeletFile() to see if it is as SLOW as Close(Report) ... it is for me')
            BUTTON('&View / Purge<13,10>Temp Folder'),AT(286,58,60,24),USE(?TempFolderBtn)
            LIST,AT(6,98),FULL,USE(?LIST:ResultsQ),NOBAR,VSCROLL,FROM(ResultsQ),FORMAT('40R(2)|M~Time<13><10>of day~C(0)' & |
                    '@t6@26R(2)|M~Pages~C(0)@n7@[41R(8)|M~RTL Normal~L(2)@s11@40R(8)|M~API Delete~L(2)@s11@]|~Time to Cl' & |
                    'ose(Report)~20L(2)~Comments~@s99@')
        END
    END

    CODE
    Glo:NoPreview=GETINI('Config','NoPreview',0,ConfigINI) 
    Pages2Print=GETINI('Config','Pages2Print',Pages2Print,ConfigINI) 
    OPEN(WINDOW)
    0{PROP:text}=clip(0{PROP:text}) &' - Process '& Glo:ProcessID & |
                                     ' - Clarion Version ' & system{PROP:LibVersion,2} &'.'& system{PROP:LibVersion,3}
    0{PROP:MinWidth}=0{PROP:Width} ; 0{PROP:MinHeight}=0{PROP:Height} * .7
    IF COMMAND('/GO') THEN POST(EVENT:Accepted, ?ReportDeleteAPI).
    ACCEPT
        CASE EVENT()
        OF EVENT:OpenWindow   ; SELECT(?ReportNormalRTL)
        OF EVENT:CloseWindow
        END
        CASE ACCEPTED()
        OF ?ReportNormalRTL ; PrintReport(Pages2Print,0) ; ?LIST:ResultsQ{PROP:Selected}=RECORDS(ResultsQ) ; DISPLAY
        OF ?ReportDeleteAPI ; PrintReport(Pages2Print,1) ; ?LIST:ResultsQ{PROP:Selected}=RECORDS(ResultsQ) ; DISPLAY
        OF ?TempFolderBtn   ; START(TempFolderPurgeTmpFiles)
        END
    END
    CLOSE(WINDOW)
    PUTINI('Config','NoPreview',Glo:NoPreview,ConfigINI)
    PUTINI('Config','Pages2Print',Pages2Print,ConfigINI)
!========================================================================= Report Test =============
! Based on Report Test from www.clarionhub.com/t/report-item-width-and-height/3629 
PrintReport PROCEDURE(Long Pages2Print, BOOL UseApiDelete)
PreviewQ  QUEUE,PRE(PreQ)
FileName    STRING(260)
    END
PrintIdx    LONG 
TimeStamp   LONG 
TimeBefore  LONG    
TimeAfter   LONG    
TimeTaken   LONG    
TakenHMSH   STRING('hh:mm:ss.hh')   !actually seconds ####.##

Report REPORT,AT(1000,1000,6250,7688),FONT('Arial',10),PRE(RPT),PAPER(PAPER:A4),THOUS
        HEADER,AT(1000,500,6500,500),USE(?HEADER1)
            STRING('Test Close(Report) Slow'),AT(208,150),USE(?Title),TRN,FONT(,,,FONT:bold)
            STRING('Process ID'),AT(4677,10),USE(?ProcessID)
            STRING('Page:'),AT(4677,177),USE(?PageLit)
            STRING(@n7),AT(5104,177),PAGENO,USE(?Page)
            STRING(@t4),AT(5854,177),USE(TimeStamp)
        END
Detail  DETAIL,AT(0,0,6250,594),USE(?Detail)
            STRING('Test Line'),AT(1646,167),USE(?TestLineText),FONT(,16)
            STRING(@n7),AT(2812,167),USE(PrintIdx),FONT(,16)
        END
    END

Progress:Thermometer LONG
ProgressWindow WINDOW('Progress...'),AT(,,192,59),CENTER,GRAY,FONT('Segoe UI',9),DOUBLE
        PROGRESS,AT(15,15,162,12),USE(Progress:Thermometer),RANGE(0,100)
        STRING(''),AT(0,3,190,10),USE(?Progress:UserString),CENTER
        STRING(''),AT(0,30,190,10),USE(?Progress:PctText),CENTER
        BUTTON('Cancel'),AT(71,43,50,15),USE(?Progress:Cancel)
    END
ForcePreview BOOL
    CODE   
    OPEN(ProgressWindow)
    ?Progress:UserString{PROP:TExt}='Printing '& Pages2Print &' Pages' ; DISPLAY
    
    OPEN(Report)
    REPORT{PROP:Preview}=PreviewQ 
    Report$?Title{PROP:Text}=Report$?Title{PROP:Text} &' - Build ' & System{PROP:LibVersion,3} !&' - Process '& Glo:ProcessID
    Report$?ProcessID{PROP:Text}='Process '& Glo:ProcessID
    ?Progress:Thermometer{PROP:RangeHigh}=Pages2Print
    0{PROP:Timer}=1 ; Progress:Thermometer=0
    PrintIdx=0 ; TimeStamp = CLOCK()
    ACCEPT                                        !Print the Report Pages 
        CASE EVENT()
        OF EVENT:Accepted
           IF FIELD()=?Progress:Cancel THEN BREAK.
        OF EVENT:Timer
           LOOP 50 TIMES
               PrintIdx += 1 
               PRINT(RPT:Detail)  
               ENDPAGE(Report)
           WHILE PrintIdx < Pages2Print
           Progress:Thermometer=PrintIdx ; DISPLAY
           IF PrintIdx >= Pages2Print THEN BREAK.
           TimeStamp = CLOCK()
        END
    END !Accept
    ENDPAGE(Report)

    !--Preview the report
    ProgressWindow{Prop:Hide}=1 ; DISPLAY    !Report template hides Progress window during preview
    IF CheckForPagesLOSTorDUP(PreviewQ) THEN ForcePreview=1.
    ReportPreviewer(PreviewQ,ForcePreview)

    !I changed Report Template like below to unhide Progress window so user knows we're busy in Close(Report) for 5 to 20 seconds
    ?Progress:UserString{PROP:TExt}='Closing Report ... Cleanup '& RECORDS(PreviewQ) &' pages'
    ?Progress:PctText{PROP:Text}='Please Wait ...'
    HIDE(?Progress:Cancel)
    ProgressWindow{Prop:Hide}=0
    DISPLAY
    
    TimeBefore = CLOCK()            !-- Time the Close(Report) without or with first deleting the Peview files using API DeleteFile
    IF UseApiDelete THEN 
       DO ApiDeletePreviewFiles     !-- Delete Preview files before ?
    END 
    CLOSE(Report)                   !-- RTL will Delete the Preview Q Temp files here, slowwww... ?   
    TimeAfter  = CLOCK()
    DO SaveTimesToResultsQ
    CLOSE(ProgressWindow)
    RETURN

ApiDeletePreviewFiles ROUTINE       !--- Delete all the files in the Preview Queue -----------------
    ?Progress:PctText{PROP:Text}=CHOOSE(~Glo:NoApiDeleteUseRTL,'API Delete PreviewQ','RTL Remove PreviewQ') ; DISPLAY
    CASE Glo:NoApiDeleteUseRTL
    OF False ; ReportCloseTempDeleteAPI(PreviewQ)  !Use API DeleteFile that is fast !!!
    OF True  ; ReportCloseTempRemoveRTL(PreviewQ)  !Use RTL Remove to see it is the slow problem
    END
    IF Glo:AfterDeleteViewTemp THEN 
       ?Progress:PctText{PROP:Text}='View Temp Folder to confirm Delete'
       TempFolderPurgeTmpFiles()
    END
    ?Progress:PctText{PROP:Text}='CLOSE(Report)'
    EXIT

SaveTimesToResultsQ ROUTINE     !--- Report is Closed so save time stats in Results Queue ---------
    TimeTaken = TimeAfter - TimeBefore + 1
    TakenHMSH = LEFT(FORMAT((TimeTaken-1)/100,@n9.2))
    
    CLEAR(ResultsQ)
    RezQ:TimeOfDay  = TimeBefore
    RezQ:Pages      = Pages2Print
    IF UseApiDelete THEN              !Was it API deleted before Close(Report) ... or RTL
       RezQ:Api_Time   = TimeTaken
       RezQ:Api_Taken  = TakenHMSH
       RezQ:Comments   = CHOOSE(~Glo:NoApiDeleteUseRTL,'2. API DeleteFile()','2. RTL Remove()')
       IF Last_Rtl_Pages = Pages2Print AND TimeTaken <= Last_Rtl_Time THEN
          RezQ:Comments   = CLIP(RezQ:Comments) &' -- '& (Last_Rtl_Time-TimeTaken)/100 &' seconds faster than RTL = '& ROUND(Last_Rtl_Time/TimeTaken,.1) &'x faster'
       ELSIF Last_Rtl_Pages = Pages2Print AND TimeTaken > Last_Rtl_Time THEN
          RezQ:Comments   = CLIP(RezQ:Comments) &' -- SLOWER than RTL ' & (TimeTaken-Last_Rtl_Time)/100 &' seconds, '& ROUND(TimeTaken/Last_Rtl_Time,.1) &'%'
       END 
    ELSE                              !This is for Normal RTL Close(Report)
       RezQ:Rtl_Time  = TimeTaken
       RezQ:Rtl_Taken = TakenHMSH
       RezQ:Comments  = '1. RTL Close(Report) Only'
       Last_Rtl_Time  = TimeTaken            
       Last_Rtl_Pages = Pages2Print       
    END
    ADD(ResultsQ)
    EXIT 
!-----------------  Open Preview Window and display --------------------------
ReportPreviewer PROCEDURE(QUEUE PreviewQ, BOOL ForcePreview)
!ReportPreviewRtn    ROUTINE     
!    DATA
PrintIt     Bool
SelectPage  LONG 
ShowPageNo  LONG
PreviewWindow WINDOW('Preview'),AT(,,440,300),CENTER,SYSTEM,MAX,ICON(ICON:Print1),FONT('Segoe UI',9,,FONT:regular), |
            ALRT(PgDnKey), ALRT(PgUpKey), ALRT(CtrlPgDn), ALRT(CtrlPgUp),RESIZE
        LIST,AT(1,2,242,11),USE(?List:PreviewQ),VSCROLL,FONT(,10),DROP(25),FROM(PreviewQ)
        ENTRY(@n_5),AT(249,1,29),USE(ShowPageNo),SKIP,TIP('Show Page Number'),ALRT(EnterKey)
        BUTTON('&Close'),AT(284,1,27,11),USE(?CloseBtn),STD(STD:Close)
        BUTTON('View Temp'),AT(318,1,43,11),USE(?ViewTempBtn),TIP('View Temp Folder')
        BUTTON('Free Q'),AT(363,1,30,11),USE(?FreeQBtn),TIP('Empty Preview Queue to prove that Close(Report) still delet' & |
                'es files<13,10>It must have a shadow copy of the queue.')
        BUTTON('P'),AT(424,1,14,11),USE(?PrintBtn),DISABLE,HIDE,TIP('Never want to print 1000s of test pages')
        BUTTON('Halt'),AT(396,1,23,11),USE(?HaltBtn),TIP('Halt preview to leave temp files in temp folder')
        IMAGE,AT(1,15),FULL,USE(?Image1),HVSCROLL
    END
    CODE
    
    OPEN(PreviewWindow)   
    0{PROP:Text}='Report Close Slow Test - Preview '& RECORDS(PreviewQ) &' Pages - Process '& Glo:ProcessID
    ?List:PreviewQ{PROP:Selected}=1
    ShowPageNo=1
    ?PrintBtn{PROP:Tip} = 'Print to ' & PRINTER{PROPPRINT:Device}
    ACCEPT
        IF ACCEPTED()=?List:PreviewQ OR EVENT()=Event:OpenWindow
           ShowPageNo = CHOICE(?List:PreviewQ)
           GET(PreviewQ,ShowPageNo)
           ?Image1{PROP:Text}=PreviewQ
           DISPLAY 
           IF Glo:NoPreview AND ~ForcePreview THEN BREAK.
        ELSIF ACCEPTED()=?ShowPageNo
           GET(PreviewQ,ShowPageNo)
           IF ~ERRORCODE() THEN
              SELECT(?List:PreviewQ,ShowPageNo)
           END 
           POST(EVENT:Accepted,?List:PreviewQ)
        ELSE 
           CASE EVENT()
           OF EVENT:AlertKey
              SelectPage=0
              CASE KEYCODE()
              OF PgDnKey  ; SelectPage=CHOICE(?List:PreviewQ)+1 
              OF PgUpKey  ; SelectPage=CHOICE(?List:PreviewQ)-1 
              OF CtrlPgDn ; SelectPage=RECORDS(PreviewQ) 
              OF CtrlPgUp ; SelectPage=1
              OF EnterKey ; UPDATE ; IF FIELD()=?ShowPageNo THEN POST(EVENT:Accepted,?ShowPageNo).
              END 
              !Debug 0{PROP:Text}='Key=' & KEYCODE() &'  SelectPage=' & SelectPage 
              IF SelectPage >= 1 AND SelectPage <= RECORDS(PreviewQ)
                 ?List:PreviewQ{PROP:Selected}=SelectPage
                 POST(EVENT:Accepted,?List:PreviewQ)
              END 
           END 
           CASE ACCEPTED() 
           OF ?PrintBtn ; PrintIt=True ; BREAK
           OF ?FreeQBtn ; FREE(PreviewQ) ; DISPLAY ; 0{PROP:Text}='Free( PreviewQ ) Done '
           OF ?ViewTempBtn ; TempFolderPurgeTmpFiles()
           OF ?HaltBtn 
                STOP('Press ABORT to Halt the program and leave WMF files in Temp folder.' &|
                     '<13,10,13,10>Use the "View / Purge Temp Folder" button to clean it out.')
           END 
        END
    END
    CLOSE(PreviewWindow)
!No    IF PrintIt THEN REPORT{PROP:FlushPreview}=True.

!===============================
!TODO: This assumes the Preview Queue is defined as just a STRING()
!      It should allow passing a <*STRING PrvQ_FileName> if you use the ability to extend the preview q with extra fields 
!============================================================================================
ReportCloseTempDeleteAPI   PROCEDURE(QUEUE RptPreviewQ)  !-- Delete Preview files using API before Close(Report) does it slowly
!============================================================================================
PrvQX      LONG   
cWmfName CSTRING(261),AUTO   
  CODE
  LOOP PrvQX=1 TO RECORDS(RptPreviewQ)
     GET(RptPreviewQ,PrvQX)
     cWmfName=CLIP(RptPreviewQ) 
     IF YES_Delete_Files THEN DeleteFile(cWmfName).  !Don't care if this fails, Report close will handle it
  END
  RETURN
ReportCloseTempRemoveRTL   PROCEDURE(QUEUE RptPreviewQ)  !-- Delete Preview files using RTL Remove() to prove as slow as Close(Report)
!============================================================================================
PrvQX      LONG   
sWmfName STRING(260),AUTO   
  CODE
  LOOP PrvQX=1 TO RECORDS(RptPreviewQ)
     GET(RptPreviewQ,PrvQX)
     sWmfName=CLIP(RptPreviewQ) 
     IF YES_Delete_Files THEN REMOVE(sWmfName, REMOVE:FILESONLY).  !Test speed of REMOVE vs DeleteFileAPI() 
  END
  RETURN

!===============================
Hex8 PROCEDURE(LONG Lng)!,STRING
LngAdj  LONG,AUTO,STATIC 
L       BYTE,DIM(4),OVER(LngAdj)
Hex     STRING('0123456789ABCDEF'),STATIC
HX      STRING(9),AUTO,STATIC 
  CODE
  LngAdj = BAND(BSHIFT(Lng, -4),0F0F0F0Fh) + 01010101h
  HX[1]=HEX[L[4]] ; HX[3]=HEX[L[3]] ; HX[5]=HEX[L[2]] ; HX[7]=HEX[L[1]]
  LngAdj=BAND(Lng,0F0F0F0Fh)  + 01010101h
  HX[2]=HEX[L[4]] ; HX[4]=HEX[L[3]] ; HX[6]=HEX[L[2]] ; HX[8]=HEX[L[1]] ; HX[9]='h'
  RETURN HX    
!===============================
DB   PROCEDURE(STRING xMessage)
Prfx EQUATE('Scratch: ')
sz   CSTRING(SIZE(Prfx)+SIZE(xMessage)+1),AUTO
  CODE 
  sz  = Prfx & CLIP(xMessage)
  OutputDebugString( sz )
!------------------
DBClear PROCEDURE()
DbgClear CSTRING('DBGVIEWCLEAR')    !Message to Clear the buffer. Must UPPER and first i.e. without a Prefix
    CODE 
    OutputDebugString(DbgClear)     !Cannot have Prefix, must be first .. so call API directly  
    
!######################################################################################
! Below code from: https://github.com/CarlTBarnes/Temp-Clean-Clarion
! This has been modified, best to take the GitHub code
!######################################################################################
!============================================================================================
TempFolderPurgeTmpFiles    PROCEDURE()   ! View Windows Temp Folder Report files and Purge them
!============================================================================================
IsVIEW  BYTE(1) !/View = Pause to view files with GO (Purge) button, also waits at end
IsWAIT  BYTE(1) !/wait = Purge but do NOT close at end 

WinTempBS  CSTRING(256)
cFileName  CSTRING(261)

QNdx    LONG,AUTO
DirQ    QUEUE(FILE:Queue),PRE(DirQ) . ! DirQ:Name  DirQ:ShortName(8.3?)  DirQ:Date  DirQ:Time  DirQ:Size  DirQ:Attrib

SortNow     SHORT(1)
SortLast    SHORT
SortNowWho  STRING(128)
SortLastWho STRING(128)
ProgNdx     LONG
ProgPct     LONG
ProgPctNew  LONG
ProgRecords LONG
ProgRatio   REAL
UseApiDelete    BYTE(1),STATIC
 
FilesWindow WINDOW('Temp Folder Cleanup'),AT(,,320,160),CENTER,GRAY,SYSTEM,ICON('CANPRINT.ICO'), |
            FONT('Segoe UI',9,,FONT:regular),RESIZE
        BUTTON('&Close'),AT(1,1,25,12),USE(?CLoseBtn),STD(STD:Close)
        BUTTON('&Refresh'),AT(30,1,26,12),USE(?RefreshBtn),SKIP,TIP('Reload Files')
        BUTTON('&Explore'),AT(59,1,28,12),USE(?ExploreBtn),SKIP,TIP('Open Temp folder in Windows Explorer')
        BUTTON('&Purge'),AT(91,1,25,12),USE(?PurgeBtn),SKIP,TIP('Delete the below files')
        CHECK('API Delete'),AT(120,1,,12),USE(UseApiDelete),SKIP,TIP('Uncheck to use RTL Remove() th' & |
                'at is SLOW')
        PROGRESS,AT(167,3,151,9),USE(ProgPct),RANGE(0,25)
        LIST,AT(1,17),FULL,USE(?ListFiles),VSCROLL,TIP('Click heads to sort, Right Click to reverse,' & |
                ' Ctrl+C to Copy'),FROM(DirQ),FORMAT('138L(1)|M~Name~@s255@52L(1)|M~EXT~@s13@40R(1)|' & |
                'M~Date~C(0)@d1@40R(1)|M~Time~C(0)@T4b@46R(1)|M~Size~C(0)@n13@')
    END
    
Dir2Clp     ANY
DateCutoff  LONG
LenFN       USHORT
Exten       STRING(4) 
MX LONG 
MY LONG 
MW LONG 
PurgeBegan LONG 
PurgeEnded LONG 
PurgeCnt   LONG 
    CODE
    IF ~GetTempPath(SIZE(WinTempBS),WinTempBS) |
    OR ~WinTempBS OR ~EXISTS(WinTempBS) THEN 
        Message('GetTempPath() failed or folder does not exist||WinTempBS='& WinTempBS &'||Last Error: ' & GetLastError(),'TempPurgeTmp',ICON:Exclamation)
        RETURN        
    END 
    DateCutoff=TODAY() -4 +4 +1 !Origial did 4 days old files, 
    DB('TempFolder: ' & WinTempBS & '  YES_Delete_Files=' & YES_Delete_Files )
    DB('DateCutoff: ' & FORMAT(DateCutoff,@d2))

    !===== Test Window to View Directory() ========================================
    SYSTEM{PROP:PropVScroll}=1
    IF THREAD() = 1 THEN GETPOSITION(0,MX,MY) ; MX += 10 ; ; MY += 10.
    OPEN(FilesWindow)
    ?ListFiles{PROP:Alrt,255}=MouseLeft
    DISPLAY              
    IF THREAD() > 1 THEN 
        SETTARGET(,1) 
        GETPOSITION(0,MX,MY,MW)  ; MX += MW
        SETTARGET() 
    END 
    SETPOSITION(0,MX,MY)
    DO LoadTempFolderToDirQRtn
    
    IF IsVIEW THEN 
       UNHIDE(?PurgeBtn)
    ELSE
       0{PROP:Timer}=1
    END
    ?ListFiles{PROP:PropVScroll}=1

    IF YES_Delete_Files=0 THEN 
        0{PROP:Text}='0=YES_Delete - ' & 0{PROP:Text}
        ?ListFiles{PROP:Background}=80000018h   !Color if Testing
        ?ListFiles{PROP:Tip}='Files with NOT really be deleted'
    END

    ACCEPT
       CASE ACCEPTED()
       OF ?PurgeBtn
            ProgRecords=RECORDS(DirQ)
            ProgRatio=?ProgPct{PROP:RangeHigh} / ProgRecords
            PurgeCnt=0
            PurgeBegan=CLOCK()
            ?ListFiles{PROP:Selected}=1
            0{PROP:Timer}=1
       OF ?RefreshBtn
            ProgPct = 0
            DO LoadTempFolderToDirQRtn
            DISPLAY
       OF ?ExploreBtn
            RUN('Explorer.exe /e,"' & CLIP(WinTempBS) &'"') 
       END
       CASE EVENT()
       OF EVENT:Timer 
          LOOP 30 TIMES
                GET(DIRQ,ProgNdx+1) 
                IF ERRORCODE() THEN 
                   0{PRop:Timer}=0
                   ?ListFiles{PROP:Selected}=ProgNdx                                
                   PurgeEnded=CLOCK() 
                   0{PROP:Text}='Purged '& PurgeCnt &' files in '& (PurgeEnded-PurgeBegan)/100 & ' seconds using ' & |
                                CHOOSE(~UseApiDelete,'RTL Remove()','API DeleteFile()') 
                   DISPLAY
                   IF ~IsWAIT AND ~IsView THEN POST(EVENT:CloseWindow) ELSE SELECT(?RefreshBtn).
                   BREAK
                END 
                ProgNdx+=1
                cFileName=WinTempBS & clip(DirQ:Name) 

                IF 0 = YES_Delete_Files THEN
                    DB('Did NOT DeleteFile: ' & DirQ:Name)
                    DirQ:ShortName='0=YES Delete' ; PUT(DIRQ) ; CYCLE
                    CYCLE 
                END 

                IF BAND(DirQ.Attrib,ff_:DIRECTORY) THEN CYCLE. !Skip folders. Original code did try to RemoveDirectory() for  Empty Folders 
                PurgeCnt += 1
                IF UseApiDelete
                   IF ~DeleteFile(cFileName) THEN       !FYI only get here if YES_Delete_Files=True
                      DirQ:ShortName='Error ' & GetLastError() ; PUT(DIRQ)
                      ?ListFiles{PROP:Selected}=ProgNdx
                      DISPLAY
                   END 
                ELSE 
                   REMOVE(cFileName, REMOVE:FILESONLY)
                   IF ERRORCODE() THEN 
                      DirQ:ShortName='Error ' & ErrorCode() ; PUT(DIRQ)
                      ?ListFiles{PROP:Selected}=ProgNdx
                      DISPLAY
                   END 
                END 
          END
          ProgPctNew = ProgNdx * ProgRatio
          IF ProgPctNew > ProgPct THEN ProgPct=ProgPctNew ; DISPLAY(?ProgPct).
!          IF 0{PRop:Timer}=0 THEN
!             SELECT(?RefreshBtn) 
!             IF ~IsWAIT AND ~IsView THEN POST(EVENT:CloseWindow).
!          END
       END 
       CASE FIELD()
       OF ?ListFiles
          CASE EVENT() !Click Header to Sort, Right Click Reverses
          OF EVENT:NewSelection !DblClick to View 1 File
             IF KEYCODE()=MouseLeft2 THEN
                GET(DirQ,CHOICE(?ListFiles))
                CASE MESSAGE(DirQ:Name &'||Exten: <9>'& DirQ:ShortName &|
                        '|Date: <9>'& FORMAT(DirQ:Date,@d02) &'|Time: <9>'& FORMAT(DirQ:Time,@t6) &|
                        '|Size: <9>'& DirQ:Size &'|Attrib: <9>'& DirQ:Attrib,|
                        'File '&CHOICE(?ListFiles),,'Close|Explore|Copy Name',,MSGMODE:CANCOPY)
                OF 2 ; RUN('Explorer.exe /select,"' & CLIP(WinTempBS & clip(DirQ:Name)) &'"') 
                OF 3 ; SETCLIPBOARD(CLIP(WinTempBS & clip(DirQ:Name))) 
                END
             END
          OF EVENT:PreAlertKey
             IF ?ListFiles{PROPList:MouseDownRow} > 0 THEN CYCLE.
             IF ?ListFiles{PROPList:MouseDownZone} = LISTZONE:right THEN CYCLE.
          !Below should be changed to use C9 Column Sorting             
          OF EVENT:AlertKey
             IF ?ListFiles{PROPList:MouseDownZone} = LISTZONE:right THEN CYCLE.
         
             SortNow=?ListFiles{PROPList:MouseDownField}
             IF ?ListFiles{PROPList:MouseDownRow} = 0  AND SortNow THEN
                IF SortNow<>ABS(SortLast) THEN SortLastWho=',' & SortNowWho.
                SortNowWho=CHOOSE(SortNow=SortLast,'-','+') & WHO(DirQ,SortNow)
                0{PROP:Text}='Directory SORT (' & SortNow &' /'& SortLast &') ' & CLIP(SortNowWho) & SortLastWho
                SORT(DirQ,CLIP(SortNowWho) & SortLastWho)
                SortLast = CHOOSE(SortNow=ABS(SortLast),-1*SortLast,SortNow)
                DISPLAY
             END
          END ! Case EVENT()
       END ! Case FIELD()
    END !ACCEPT
    CLOSE(FilesWindow)
    RETURN                                                                                   

LoadTempFolderToDirQRtn ROUTINE
    FREE(DirQ)
    DIRECTORY(DirQ,WinTempBS&'*.tmp',ff_:NORMAL)
    DIRECTORY(DirQ,WinTempBS&'*.wmf',ff_:NORMAL)

    LOOP QNdx = RECORDS(DirQ) TO 1 BY -1
         GET(DirQ,QNdx)
             DirQ:Name=LOWER(DirQ:Name)
             LenFN=LEN(CLIP(DirQ:Name))
             Exten=SUB(DirQ:Name,LenFn-3,4) ; DirQ:shortname=Exten
             PUT(DirQ) 

         IF BAND(DirQ:Attrib,FF_:Directory) OR DirQ:Name='.' OR DirQ:Name='..'   
            DELETE(DirQ)  !skip . or .. Dirs

         ELSIF DirQ:Date > DateCutoff THEN
            !DirQ:Name=' date ' & DirQ:Name ; PUT(DirQ)     !see Date cutoff in list 
            DELETE(DirQ)

         ELSE
             CASE Exten
             OF '.wmf'  !Delete  Zero Byte .WMF OR 12345678.WMF Created by CPCS
                    IF DirQ:Name[1:3]='cla' THEN
                        !In C11.1 seeing Cla.WMF files?YES confirmed by Graham Dawson was done first 11.1.13744
                    ELSIF ~NUMERIC(DirQ:Name[1:8]) AND DirQ:Size THEN
                        DirQ:Name=' skip ' & DirQ:Name              !Only Delete CPCS or Zero Byte
                    END
             OF '.tmp'
                    IF DirQ:Name[1:3]='pdf' OR DirQ:Name[1:3]='cla' THEN    !Tracker PDF####.Tmp or CLA###.Tmp Temp Report
                    ELSE    
                       DirQ:Name=' skip ' & DirQ:Name
                    END 
             END             
             IF ~DirQ:Name[1] THEN   !is [1] Blank ' skip'  !Change to [2] to test other files
                !    PUT(DirQ)               !Test so Skip in List
                DELETE(DirQ)
             ELSE
                PUT(DirQ)
             END
         END
    END !LOOP Delete files
    SortNow=1
    SortNowWho=WHO(DirQ,SortNow) 
    SORT(DirQ , DirQ:Name)  !  DirQ:Name , DirQ:ShortName , DirQ:Date , DirQ:Time , DirQ:Size , DirQ:Attrib )    
!    0{PROP:Text}='Temp Cleanup '& WinTempBS & ' (' & RECORDS(DirQ) &' records) ' 
    0{PROP:Text}='Temp Cleanup - ' & RECORDS(DirQ) &' files - '  & WinTempBS
    EXIT
!###########################################################################
CheckForPagesLOSTorDUP      PROCEDURE(QUEUE PreviewQ)!,BOOL,PROC
PrevQ:FileName     PSTRING(256) 
DupCheckQ QUEUE,PRE(DupChkQ)
PageNo      LONG            !DupChkQ:PageNo
FileName    STRING(260)     !DupChkQ:FileName
    END
PgX     LONG
BadMsg  ANY
BadCnt  LONG
    CODE
    LOOP PgX=1 TO RECORDS(PreviewQ)
         GET(PreviewQ,PgX)
         PrevQ:FileName = CLIP(PreviewQ)
         IF ~EXISTS(PrevQ:FileName)       !Test OR PgX=33   !Problem: File not on disk?
            BadMsg=BadMsg &'<13,10>Page '& PgX &' file not found: '& PrevQ:FileName
            BadCnt += 1
         END

         DupChkQ:FileName=UPPER(PrevQ:FileName)
         GET(DupCheckQ,DupChkQ:FileName)                !Check if WMF Name for already in DupQ i.e. Duplicate
         IF ~ERRORCODE()                  !Test OR PgX % 22 = 0
            BadMsg=BadMsg &'<13,10>Pages '& PgX &' and '& DupChkQ:PageNo &' same: '& PrevQ:FileName
            BadCnt += 1
         ELSE
            DupChkQ:PageNo = PgX
            ADD(DupCheckQ,DupChkQ:FileName)
         END
    END
    IF ~BadCnt THEN RETURN False.
    
    BadMsg=BadCnt &' problems found in '& RECORDS(PreviewQ) &' page report:<13,10>'& BadMsg
    CASE Message(BadMsg,'Report Pages Lost or Duplicated - Process ' & Glo:ProcessID,ICON:NoPrint,'Close|Copy Text',,MSGMODE:CANCOPY)
    OF 2 ; SETCLIPBOARD(BadMsg)
    END
    RETURN True
